local thought_warrior = Talents:getTalentFromId(Talents.T_TF_WARRIOR)
thought_warrior.activate = function(self, t)
	cancelThoughtForms(self, t.id)
	
	-- Find space
	local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
	if not x then
		game.logPlayer(self, "Not enough space to summon!")
		return
	end
	
	-- Do our stat bonuses here so we only roll for crit once		
	local stat_bonus = math.floor(self:mindCrit(t.getStatBonus(self, t)))

	local NPC = require "mod.class.NPC"
	local m = NPC.new{
		name = "thought-forged warrior", summoner = self, 
		color=colors.ORANGE, shader = "shadow_simulacrum",
		shader_args = { color = {0.8, 0.8, 0.8}, base = 0.8, time_factor = 4000 },
		desc = [[A thought-forged warrior wielding a massive hammer and clad in heavy armor.  It appears ready for battle.]],
		body = { INVEN = 10, MAINHAND = 1, BODY = 1, HANDS = 1, FEET = 1},
	
		ai = "summoned", ai_real = "tactical",
		ai_state = { ai_move="move_dmap", talent_in=3, ally_compassion=10 },
		ai_tactic = resolvers.tactic("melee"),
		
		max_life = resolvers.rngavg(100,110),
		life_rating = 15,
		combat_armor = 0, combat_def = 0,
		stats = { mag=self:getMag(), wil=self:getWil(), cun=self:getCun()},
		inc_stats = {
			str = stat_bonus,
			dex = stat_bonus / 2,
			con = stat_bonus / 2,
		},

		resolvers.generic(function(e) buildTile(e) end), -- Make a moddable tile
		resolvers.talents{ 
			[Talents.T_ARMOUR_TRAINING]= 2,
			[Talents.T_WEAPON_COMBAT]= math.ceil(self.level/10),
			[Talents.T_WEAPONS_MASTERY]= math.ceil(self.level/10),
			
			[Talents.T_RUSH]= math.ceil(self.level/10),
			[Talents.T_DEATH_DANCE]= math.ceil(self.level/10),
			[Talents.T_BERSERKER]= math.ceil(self.level/10),

			[Talents.T_PSYCHOMETRY]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),
			[Talents.T_BIOFEEDBACK]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),
			[Talents.T_LUCID_DREAMER]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),
		},
		resolvers.equip{
			{type="weapon", subtype="battleaxe", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="heavy", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="hands", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="feet", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
		},
		resolvers.sustains_at_birth(),
	}

	setupThoughtForm(self, m, x, y, t)

	game:playSoundNear(self, "talents/spell_generic")
	
	local ret = {
		summon = m
	}
	if self:knowTalent(self.T_TF_UNITY) then
		local t = self:getTalentFromId(self.T_TF_UNITY)
		ret.power = self:addTemporaryValue("combat_mindpower", t.getOffensePower(self, t))
	end
	return ret
end

local thought_defender = Talents:getTalentFromId(Talents.T_TF_DEFENDER)
thought_defender.activate = function(self, t)
	cancelThoughtForms(self, t.id)
	
	-- Find space
	local x, y = util.findFreeGrid(self.x, self.y, 5, true, {[Map.ACTOR]=true})
	if not x then
		game.logPlayer(self, "Not enough space to summon!")
		return
	end
	
	-- Do our stat bonuses here so we only roll for crit once	
	local stat_bonus = math.floor(self:mindCrit(t.getStatBonus(self, t)))

	local NPC = require "mod.class.NPC"
	local m = NPC.new{
		name = "thought-forged defender", summoner = self,
		color=colors.GOLD, shader = "shadow_simulacrum", 
		shader_args = { color = {0.8, 0.8, 0.8}, base = 0.8, time_factor = 4000 },
		desc = [[A thought-forged defender clad in massive armor.  It wields a sword and shield and appears ready for battle.]],
		body = { INVEN = 10, MAINHAND = 1, OFFHAND = 1, BODY = 1, HANDS = 1, FEET = 1},
		
		ai = "summoned", ai_real = "tactical",
		ai_state = { ai_move="move_dmap", talent_in=3, ally_compassion=10 },
		ai_tactic = resolvers.tactic("tank"),
		
		max_life = resolvers.rngavg(100,110),
		life_rating = 15,
		combat_armor = 0, combat_def = 0,
		stats = { mag=self:getMag(), wil=self:getWil(), cun=self:getCun()},
		inc_stats = {
			str = stat_bonus / 2,
			dex = stat_bonus / 2,
			con = stat_bonus,
		},
		
		resolvers.generic(function(e) buildTile(e) end), -- Make a moddable tile
		resolvers.talents{ 
			[Talents.T_ARMOUR_TRAINING]= 2 + math.ceil(self.level/20),
			[Talents.T_WEAPON_COMBAT]= math.ceil(self.level/10),
			[Talents.T_WEAPONS_MASTERY]= math.ceil(self.level/10),
			
			[Talents.T_SHIELD_PUMMEL]= math.ceil(self.level/10),
			[Talents.T_SHIELD_WALL]= math.ceil(self.level/10),
			

			[Talents.T_PSYCHOMETRY]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),
			[Talents.T_BIOFEEDBACK]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),
			[Talents.T_LUCID_DREAMER]= math.floor(self:getTalentLevel(self.T_TRANSCENDENT_THOUGHT_FORMS)),

		},
		resolvers.equip{
			{type="weapon", subtype="longsword", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="shield", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="massive", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="hands", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
			{type="armor", subtype="feet", autoreq=true, forbid_power_source={arcane=true}, not_properties = {"unique"} },
		},
		resolvers.sustains_at_birth(),
	}

	setupThoughtForm(self, m, x, y, t)

	game:playSoundNear(self, "talents/spell_generic")
	
	local ret = {
		summon = m
	}
	if self:knowTalent(self.T_TF_UNITY) then
		local t = self:getTalentFromId(self.T_TF_UNITY)
		ret.resist = self:addTemporaryValue("resists", {all= t.getDefensePower(self, t)})
	end
	return ret
end
