local class        = require 'engine.class'
local Birther      = require 'engine.Birther'
local ActorTalents = require 'engine.interface.ActorTalents'
local DamageType   = require 'engine.DamageType'

class:bindHook("ToME:load", function(self, data)
	-- Correct the birth talent levels, if necessary
	local birth_fixups = {
		["Sun Paladin"] = 2,
		["Tutorial Adventurer"] = 2,
		["Berserker"] = 1,
		["Bulwark"] = 2,
	}
	for subclass, at_level in pairs(birth_fixups) do
		local birther = Birther:getBirthDescriptor("subclass", subclass)
		if birther then birther.talents[ActorTalents.T_ARMOUR_TRAINING] = at_level end
	end
	
	-- Fix up the only damage type that uses Armour Training
	local dam_type = DamageType:get(DamageType.GARKUL_INVOKE)
	if dam_type then
		dam_type.projector = function(src, x, y, type, dam)
			if not rng.percent(dam) then return end
			local target = game.level.map(x, y, engine.Map.ACTOR)
			if not target then return end

			if game.party:hasMember(src) and game.party:findMember{type="garkul spirit"} then return end

			-- Find space
			local x, y = util.findFreeGrid(src.x, src.y, 5, true, {[engine.Map.ACTOR]=true})
			if not x then return end

			print("Invoking garkul spirit on", x, y)

			local NPC = require "mod.class.NPC"
			local orc = NPC.new{
				type = "humanoid", subtype = "orc",
				display = "o", color=colors.UMBER,
				combat = { dam=resolvers.rngavg(5,12), atk=2, apr=6, physspeed=2 },
				body = { INVEN = 10, MAINHAND=1, OFFHAND=1, BODY=1, QUIVER=1 },
				infravision = 10,
				lite = 1,
				rank = 2,
				size_category = 3,
				resolvers.racial(),
				resolvers.sustains_at_birth(),
				autolevel = "warrior",
				ai = "summoned", ai_real = "dumb_talented_simple", ai_state = { ai_move="move_dmap", talent_in=2, },
				stats = { str=20, dex=8, mag=6, con=16 },
				name = "orc spirit", color=colors.SALMON, image = "npc/humanoid_orc_orc_berserker.png",
				desc = [[An orc clad in a massive armour, wielding a huge axe.]],
				level_range = {35, nil}, exp_worth = 0,
				max_life = resolvers.rngavg(110,120), life_rating = 12,
				resolvers.equip{
					{type="weapon", subtype="battleaxe", autoreq=true},
					{type="armor", subtype="massive", autoreq=true},
				},
				combat_armor = 0, combat_def = 5,

				resolvers.talents{
					[src.T_ARMOUR_TRAINING]={base=2, every=10, max=3},
					[src.T_WEAPON_COMBAT]={base=2, every=10, max=4},
					[src.T_WEAPONS_MASTERY]={base=2, every=10, max=4},
					[src.T_RUSH]={base=3, every=7, max=6},
					[src.T_STUNNING_BLOW]={base=3, every=7, max=6},
					[src.T_BERSERKER]={base=3, every=7, max=6},
				},

				faction = src.faction,
				summoner = src,
				summon_time = 6,
			}

			orc:resolve() orc:resolve(nil, true)
			game.zone:addEntity(game.level, orc, "actor", x, y)
			orc:forceLevelup(src.level)

			orc.remove_from_party_on_death = true
			game.party:addMember(orc, {control="no", type="garkul spirit", title="Garkul Spirit"})
			orc:setTarget(target)
		end
	end
	
	-- Fix talents
	ActorTalents:loadDefinition("/data-fpat/talents/psionic/thought-forms.lua")
	ActorTalents:loadDefinition("/data-fpat/talents/spells/golem.lua")
	ActorTalents:loadDefinition("/data-fpat/talents/spells/golemancy.lua")
	ActorTalents:loadDefinition("/data-fpat/talents/spells/necrotic-minions.lua")
	ActorTalents:loadDefinition("/data-fpat/talents/techniques/combat-training.lua")
	ActorTalents:loadDefinition("/data-fpat/talents/uber/str.lua")
end)

-- This is way more complicated than it should have to be
local fixed, last_list
-- Run through the list of NPCs (entities, really) and update their AT talent resolver
local function updateNPC(npc_list)
	if npc_list ~= last_list then
		fixed, last_list = {}, npc_list
	end
	for _, npc in ipairs(npc_list) do
		if not fixed[npc] then
			local talents = nil
			-- Find talent resolver table...
			for _, whoknows in ipairs(npc) do
				if type(whoknows) == 'table' and whoknows.__resolver and whoknows.__resolver == 'talents' then
					talents = whoknows[1]
					break
				end
			end
			-- ... and fix up the AT level, if any
			if talents then
				local npc_at = talents[ActorTalents.T_ARMOUR_TRAINING]
				if type(npc_at) == 'table' then
					talents[ActorTalents.T_ARMOUR_TRAINING] = {
						base = math.ceil(npc_at.base / 2),
						every = 2 * npc_at.every,
						max = math.ceil(npc_at.max / 2)}
				elseif type(npc_at) == 'number' then
					talents[ActorTalents.T_ARMOUR_TRAINING] = math.ceil(npc_at / 2)
				end
			end
			fixed[npc] = true	-- Store the NPC reference so we don't process it again
		end
	end
end

-- Fix the AT requirements on armor
local function updateArmor(armor_list)
	local reqs = {shield = 2, massive = 3}
	for _, armor in ipairs(armor_list) do
		if armor.type == 'armor' then
			if reqs[armor.subtype] then
				armor.require.talent = {{ActorTalents.T_ARMOUR_TRAINING, reqs[armor.subtype]}}
			elseif armor.subtype == 'heavy' then
				armor.require.talent = {ActorTalents.T_ARMOUR_TRAINING}
			elseif armor.define_as == 'HELM_KROLTAR' then
				armor.require.talent = {{ActorTalents.T_ARMOUR_TRAINING, 3}}
			end
		end
	end
end
 
class:bindHook("Entity:loadList", function (self, data)
	local file = (data.file:find('/')) == 1 and data.file or '/'..data.file
	local npc_fixups = {
		['/data/general/npcs/construct.lua'] = true,				-- general npcs
		['/data/general/npcs/elven-warrior.lua'] = true,
		['/data/general/npcs/minor-demon.lua'] = true,
		['/data/general/npcs/orc-gorbat.lua'] = true,
		['/data/general/npcs/orc-grushnak.lua'] = true,
		['/data/general/npcs/orc.lua'] = true,
		['/data/general/npcs/skeleton.lua'] = true,
		['/data/general/npcs/sunwall-town.lua'] = true,
		['/data/general/npcs/ziguranth.lua'] = true,
		['/data/zones/arena/npcs.lua'] = true,					-- zone specific npcs
		['/data/zones/arena-unlock/npcs.lua'] = true,
		['/data/zones/charred-scar/npcs.lua'] = true,
		['/data/zones/daikara/npcs.lua'] = true,
		['/data/zones/deep-bellow/npcs.lua'] = true,
		['/data/zones/dreadfell/npcs.lua'] = true,
		['/data/zones/eruan/npcs.lua'] = true,
		['/data/zones/gorbat-pride/npcs.lua'] = true,
		['/data/zones/grushnak-pride/npcs.lua'] = true,
		['/data/zones/high-peak/npcs.lua'] = true,
		['/data/zones/keepsake-meadow/npcs.lua'] = true,
		['/data/zones/maze/npcs.lua'] = true,
		['/data/zones/old-forest/npcs.lua'] = true,
		['/data/zones/reknor/npcs.lua'] = true,
		['/data/zones/reknor-escape/npcs.lua'] = true,
		['/data/zones/ring-of-blood/npcs.lua'] = true,
		['/data/zones/tannen-tower/npcs.lua'] = true,
		['/data/zones/town-gates-of-morning/npcs.lua'] = true,
		['/data/zones/town-zigur/npcs.lua'] = true,
		['/data/zones/trollmire/npcs.lua'] = true,
		['/data/zones/unremarkable-cave/npcs.lua'] = true,
	}
	-- Update NPC Armour Training talents
	if npc_fixups[file] then
		updateNPC(data.res)
		return
	end

	local armor_fixups = {
		['/data/general/objects/heavy-armors.lua'] = true,
		['/data/general/objects/massive-armors.lua'] = true,
		['/data/general/objects/shields.lua'] = true,
		['/data/general/objects/world-artifacts.lua'] = true,
	}
	-- Update basic armor requirements
	if armor_fixups[file] then
		updateArmor(data.res)
		return
	end	
end)